import click
import os
import yaml

from kfk.command import kfk
from kfk.option_extensions import NotRequiredIf, RequiredIf
from kfk.commons import print_missing_options_for_command, delete_last_applied_configuration, resource_exists, \
    get_resource_as_file, add_resource_kv_config, \
    delete_resource_config
from kfk.constants import *
from kfk.kubectl_command_builder import Kubectl
from kfk.dependencies import download_strimzi_if_not_exists


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--delete-config', help='A topic configuration override to be removed for an existing topic',
              multiple=True)
@click.option('--config', help='A topic configuration override for the topic being created or altered.', multiple=True)
@click.option('--alter', help='Alter the number of partitions, replica assignment, and/or configuration of the topic.',
              is_flag=True)
@click.option('--delete', help='Delete a topic.', is_flag=True)
@click.option('--native', help='List details for the given topic natively.', is_flag=True, cls=RequiredIf,
              required_if='describe')
@click.option('-o', '--output',
              help='Output format. One of: json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath'
                   '|jsonpath-file.')
@click.option('--describe', help='List details for the given topic.', is_flag=True)
@click.option('--replication-factor', help='The replication factor for each partition in the topic being created.',
              cls=RequiredIf, required_if='create')
@click.option('--partitions', help='The number of partitions for the topic being created or altered ', cls=RequiredIf,
              required_if='create')
@click.option('--create', help='Create a new topic.', is_flag=True)
@click.option('--list', help='List all available topics.', is_flag=True)
@click.option('--topic', help='Topic Name', required=True, cls=NotRequiredIf, not_required_if='list')
@kfk.command()
def topics(topic, list, create, partitions, replication_factor, describe, output, native, delete, alter, config,
           delete_config, cluster, namespace):
    """The kafka topic(s) to be created, altered or described."""
    if list:
        os.system(
            Kubectl().get().kafkatopics().label("strimzi.io/cluster={cluster}").namespace(namespace).build().format(
                cluster=cluster))
    elif create:
        download_strimzi_if_not_exists()

        with open(r'{strimzi_path}/examples/topic/kafka-topic.yaml'.format(strimzi_path=STRIMZI_PATH).format(
                version=STRIMZI_VERSION)) as file:
            topic_dict = yaml.full_load(file)

            topic_dict["metadata"]["name"] = topic
            topic_dict["spec"]["partitions"] = int(partitions)
            topic_dict["spec"]["replicas"] = int(replication_factor)

            add_resource_kv_config(config, topic_dict)

            topic_yaml = yaml.dump(topic_dict)
            os.system(
                'echo "{topic_yaml}" | '.format(topic_yaml=topic_yaml) + Kubectl().create().from_file("-").namespace(
                    namespace).build())

    elif describe:
        if output is not None:
            if resource_exists("kafkatopics", topic, cluster, namespace):
                os.system(
                    Kubectl().get().kafkatopics(topic).namespace(namespace).output(output).build())
        else:
            if native:
                native_command = "bin/kafka-topics.sh --bootstrap-server localhost:9092 --describe --topic {topic}"
                os.system(
                    Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(namespace).exec_command(
                        native_command).build().format(topic=topic, cluster=cluster))
            else:
                if resource_exists("kafkatopics", topic, cluster, namespace):
                    os.system(
                        Kubectl().describe().kafkatopics(topic).namespace(namespace).build())

    elif delete:
        if resource_exists("kafkatopics", topic, cluster, namespace):
            os.system(Kubectl().delete().kafkatopics(topic).namespace(namespace).build())

    elif alter:
        if resource_exists("kafkatopics", topic, cluster, namespace):
            file = get_resource_as_file("kafkatopics", topic, namespace)
            topic_dict = yaml.full_load(file)

            if partitions is not None:
                topic_dict["spec"]["partitions"] = int(partitions)

            if replication_factor is not None:
                topic_dict["spec"]["replicas"] = int(replication_factor)

            delete_last_applied_configuration(topic_dict)

            if len(config) > 0:
                if topic_dict["spec"].get("config") is None:
                    topic_dict["spec"]["config"] = {}
                add_resource_kv_config(config, topic_dict["spec"]["config"])

            if len(delete_config) > 0:
                if topic_dict["spec"].get("config") is not None:
                    delete_resource_config(delete_config, topic_dict["spec"]["config"])

            topic_yaml = yaml.dump(topic_dict)
            os.system(
                'echo "{topic_yaml}" | '.format(topic_yaml=topic_yaml) + Kubectl().apply().from_file("-").namespace(
                    namespace).build())
    else:
        print_missing_options_for_command("topics")



import os
import stat
import wget
import tarfile
import ssl

from kfk.constants import *
from pathlib import Path

ssl._create_default_https_context = ssl._create_unverified_context


def download_strimzi_if_not_exists():
    if not os.path.exists(STRIMZI_PATH):
        strimzi_tarfile_path = STRIMZI_PATH + ".tar.gz"

        print(
            "Creating Strimzi Kafka CLI Dependencies folder: {base_path}\n".format(base_path=BASE_PATH))
        Path(BASE_PATH).mkdir(exist_ok=True)

        print("Downloading dependency: Strimzi {version}...\n".format(version=STRIMZI_VERSION))
        wget.download(STRIMZI_RELEASE_URL, strimzi_tarfile_path)
        print("\nDownload successfully completed!\n")
        print("Extracting Strimzi {version}...\n".format(version=STRIMZI_VERSION))
        tar = tarfile.open(strimzi_tarfile_path)
        tar.extractall(path=BASE_PATH)
        tar.close()
        os.remove(strimzi_tarfile_path)


def download_kubectl_if_not_exists():
    if not os.path.exists(KUBECTL_PATH):
        print(
            "Creating Strimzi Kafka CLI Dependencies folder: {base_path}\n".format(base_path=BASE_PATH))
        Path(BASE_PATH).mkdir(exist_ok=True)

        print("Downloading dependency: kubectl {version}...\n".format(version=KUBECTL_VERSION))
        wget.download(KUBECTL_RELEASE_URL, KUBECTL_PATH)
        print("\nDownload successfully completed!\n")
        current_stat = os.stat(KUBECTL_PATH)
        os.chmod(KUBECTL_PATH, current_stat.st_mode | stat.S_IEXEC)



import click

from kfk.dependencies import download_kubectl_if_not_exists


@click.group()
def kfk():
    """Strimzi Kafka CLI"""
    download_kubectl_if_not_exists()


if __name__ == '__main__':
    kfk(prog_name='kfk')



import os
import io

from kfk.kubectl_command_builder import Kubectl
from kfk.utils import convert_string_to_type


def print_missing_options_for_command(command_str):
    print("Missing options: kfk {command_str} [OPTIONS] \nTry 'kfk {command_str} --help' for help.".format(
        command_str=command_str))


def delete_last_applied_configuration(resource_dict):
    if "annotations" in resource_dict["metadata"]:
        del resource_dict["metadata"]["annotations"]["kubectl.kubernetes.io/last-applied-configuration"]


def add_resource_kv_config(config, dict_part):
    if type(config) is tuple:
        for config_str in config:
            config_arr = config_str.split('=')
            dict_part[config_arr[0]] = convert_string_to_type(config_arr[1])
    else:
        config_arr = config.split('=')
        dict_part[config_arr[0]] = convert_string_to_type(config_arr[1])


def delete_resource_config(config, dict_part):
    if type(config) is tuple:
        for config_str in config:
            if config_str in dict_part:
                del dict_part[config_str]
    else:
        del dict_part[config]


def resource_exists(resource_type, resource_name, cluster, namespace):
    return resource_name in os.popen(
        Kubectl().get().resource(resource_type).label("strimzi.io/cluster={cluster}").namespace(
            namespace).build().format(
            cluster=cluster)).read()


def get_resource_yaml(resource_type, resource_name, namespace):
    return os.popen(
        Kubectl().get().resource(resource_type, resource_name).namespace(namespace).output("yaml").build()).read()


def get_resource_as_file(resource_type, resource_name, namespace):
    topic_yaml = get_resource_yaml(resource_type, resource_name, namespace)
    return io.StringIO(topic_yaml)



import click
import os
import yaml

from kfk.command import kfk
from kfk.option_extensions import NotRequiredIf, RequiredIf
from kfk.commons import print_missing_options_for_command, resource_exists, get_resource_as_file, \
    delete_last_applied_configuration, add_resource_kv_config, delete_resource_config
from kfk.constants import *
from kfk.kubectl_command_builder import Kubectl
from kfk.dependencies import download_strimzi_if_not_exists


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--delete-quota', help='User quotas to be removed.', multiple=True)
@click.option('--quota', help='User\'s network and CPU utilization quotas in the Kafka cluster.', multiple=True)
@click.option('--alter', help='Alter authentication-type, quotas, etc. of the user.',
              is_flag=True)
@click.option('--delete', help='Delete a user.', is_flag=True)
@click.option('-o', '--output',
              help='Output format. One of: json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath'
                   '|jsonpath-file.')
@click.option('--describe', help='List details for the given user.', is_flag=True)
@click.option('--authentication-type', type=click.Choice(['tls', 'scram-sha-512'], case_sensitive=True), cls=RequiredIf,
              required_if='create')
@click.option('--create', help='Create a new user.', is_flag=True)
@click.option('--list', help='List all available users.', is_flag=True)
@click.option('--user', help='User Name', required=True, cls=NotRequiredIf, not_required_if='list')
@kfk.command()
def users(user, list, create, authentication_type, describe, output, delete, alter, quota, delete_quota, cluster,
          namespace):
    """The kafka user(s) to be created, altered or described."""
    if list:
        os.system(
            Kubectl().get().kafkausers().label("strimzi.io/cluster={cluster}").namespace(namespace).build().format(
                cluster=cluster))
    elif create:
        download_strimzi_if_not_exists()

        with open(r'{strimzi_path}/examples/user/kafka-user.yaml'.format(strimzi_path=STRIMZI_PATH).format(
                version=STRIMZI_VERSION)) as file:
            user_dict = yaml.full_load(file)

            user_dict["metadata"]["name"] = user
            user_dict["spec"]["authentication"]["type"] = authentication_type
            del user_dict["spec"]["authorization"]

            user_yaml = yaml.dump(user_dict)
            os.system(
                'echo "{user_yaml}" | '.format(user_yaml=user_yaml) + Kubectl().create().from_file("-").namespace(
                    namespace).build())

    elif describe:
        if output is not None:
            if resource_exists("kafkausers", user, cluster, namespace):
                os.system(Kubectl().get().kafkausers(user).namespace(namespace).output(output).build())
        else:
            if resource_exists("kafkausers", user, cluster, namespace):
                os.system(Kubectl().describe().kafkausers(user).namespace(namespace).build())
    elif delete:
        if resource_exists("kafkausers", user, cluster, namespace):
            os.system(Kubectl().delete().kafkausers(user).namespace(namespace).build())
    elif alter:
        if resource_exists("kafkausers", user, cluster, namespace):
            file = get_resource_as_file("kafkausers", user, namespace)
            user_dict = yaml.full_load(file)

            if authentication_type is not None:
                user_dict["spec"]["authentication"]["type"] = authentication_type

            delete_last_applied_configuration(user_dict)

            if len(quota) > 0:
                if user_dict["spec"].get("quotas") is None:
                    user_dict["spec"]["quotas"] = {}
                add_resource_kv_config(quota, user_dict["spec"]["quotas"])

            if len(delete_quota) > 0:
                if user_dict["spec"].get("quotas") is not None:
                    delete_resource_config(delete_quota, user_dict["spec"]["quotas"])

            topic_yaml = yaml.dump(user_dict)
            os.system(
                'echo "{topic_yaml}" | '.format(topic_yaml=topic_yaml) + Kubectl().apply().from_file("-").namespace(
                    namespace).build())
    else:
        print_missing_options_for_command("users")



import platform
import sys

from pathlib import Path

BASE_FOLDER = ".strimzi-kafka-cli"
BASE_PATH = str(Path.home()) + "/" + BASE_FOLDER
STRIMZI_VERSION = "0.18.0"
STRIMZI_PATH = BASE_PATH + "/strimzi-{version}".format(version=STRIMZI_VERSION)
STRIMZI_RELEASE_URL = "https://github.com/strimzi/strimzi-kafka-operator/releases/download/{version}/strimzi-{version}.tar.gz".format(
    version=STRIMZI_VERSION)
KUBECTL_VERSION = "v1.18.0"
KUBECTL_PATH = BASE_PATH + "/kubectl"
PROCESSOR_TYPE = 'amd64' if sys.maxsize > 2 ** 32 else "386"
KUBECTL_RELEASE_URL = "https://storage.googleapis.com/kubernetes-release/release/{version}/bin/{operating_system}/{processor_type}/kubectl".format(
    version=KUBECTL_VERSION, operating_system=platform.system().lower(), processor_type=PROCESSOR_TYPE)
SPACE = " "



import click
import os

from kfk.command import kfk
from kfk.kubectl_command_builder import Kubectl


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--topic', help='Topic Name', required=True)
@click.option('--from-beginning', help='Consumes messages from beginning', is_flag=True)
@kfk.command()
def console_consumer(topic, cluster, from_beginning, namespace):
    """The console consumer is a tool that reads data from Kafka and outputs it to standard output."""
    native_command = "bin/kafka-console-consumer.sh --bootstrap-server localhost:9092 --topic {topic} {from_beginning}"
    os.system(
        Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(namespace).exec_command(
            native_command).build().format(cluster=cluster, topic=topic,
                                           from_beginning=(from_beginning and '--from-beginning' or '')))


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--topic', help='Topic Name', required=True)
@kfk.command()
def console_producer(topic, cluster, namespace):
    """The console producer is a tool that reads data from standard input and publish it to Kafka."""
    native_command = "bin/kafka-console-producer.sh --broker-list localhost:9092 --topic {topic}"
    os.system(
        Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(namespace).exec_command(
            native_command).build().format(cluster=cluster, topic=topic))
